// ----------------------------------------------------------------------------
// Copyright (C) 2008 Francisco Grimaldo, Miguel Lozano, Fernando Barber
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
// 
// To contact the authors:
// francisco.grimaldo@uv.es, miguel.lozano@uv.es, fernando.barber@uv.es
// http://www.uv.es/grimo
// http://grev.uv.es
//
//----------------------------------------------------------------------------

package jmadem;

import jason.asSyntax.Term;

/** 
 * Interface for the filter functions invoked by MADeM agents when constructing allocations.
 * The code of every filter function must implement this interface.
 *
 * @author Francisco Grimaldo
 */
public interface FilterFunctionInt 
{
	/**
	  * Tests whether a certain allocation should be filtered out within a given domain. 
	  *
	  * @param	alloc	allocation being evaluated
	  * @param	agArch	architecture of the MADeM agent
	  * 
	  * @throws	FilterFunctionException when any error occurs
	  * @return	<tt>true</tt> if the allocation has to be filtered out, <tt>false</tt> otherwise 
	  */
    public boolean doFilter(Term alloc, MADeMAgArch agArch) throws FilterFunctionException;
}
