// ----------------------------------------------------------------------------
// Copyright (C) 2008 Francisco Grimaldo, Miguel Lozano, Fernando Barber
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
// 
// To contact the authors:
// francisco.grimaldo@uv.es, miguel.lozano@uv.es, fernando.barber@uv.es
// http://www.uv.es/grimo
// http://grev.uv.es
//
//----------------------------------------------------------------------------

package jmadem;

import jason.asSemantics.DefaultInternalAction;
import jason.asSemantics.TransitionSystem;
import jason.asSemantics.Unifier;
import jason.asSyntax.Atom;
import jason.asSyntax.Term;

import java.util.logging.Logger;

/** 
 * Internal function for removing a filter function to the MADeM agent.
 * <p>
 * <b>Use:</b> <tt>jmadem.remove_filter_function("package.FilterFunctionName")</tt>
 * </p>
 *
 * @author Francisco Grimaldo
 */
public class remove_filter_function extends DefaultInternalAction  {

	private static final long serialVersionUID = -1668039429089053537L;
	
	Logger logger = Logger.getLogger(remove_filter_function.class.getName());

	@Override
	public Object execute(TransitionSystem ts, Unifier un, Term[] args) throws Exception 
	{
		try
		{
			// Get the name of the filter function from the arguments
			Atom fFuncNameTerm = (Atom) args[0];
					
			// Remove the filter function
			MADeMAgArch agArch = (MADeMAgArch)ts.getUserAgArch();
			if (agArch.removeFilterFunction(fFuncNameTerm))
				logger.fine("Filter function " + fFuncNameTerm + "correctly removed.");
			else
				logger.warning("Filter function \"" + fFuncNameTerm + "\" can not be removed. It seems not to be registered.");
		}
		catch (Exception e)
		{
			logger.severe("Error removing filter function "+ args[0]);
			e.printStackTrace();
    	    return false;
		}
		return true;
    }
}
